<?php declare(strict_types=1);

namespace App\Service;

use App\Entity\FoodOrder;
use App\Entity\FoodVendor;
use App\Entity\OrderItem;
use App\Repository\FoodOrderRepository;
use App\Repository\FoodVendorRepository;
use App\Repository\OrderItemRepository;
use Doctrine\ORM\EntityManagerInterface;

use function range;

final readonly class FakeData
{
    public function __construct(
        private EntityManagerInterface $entityManager,
        private FoodVendorRepository $foodVendorRepository,
        private FoodOrderRepository $foodOrderRepository,
        private OrderItemRepository $orderItemRepository,
    ) {}

    public function resetDb(): void
    {
        foreach ($this->orderItemRepository->findAll() as $item) {
            $this->entityManager->remove($item);
        }
        foreach ($this->foodOrderRepository->findAll() as $item) {
            $this->entityManager->remove($item);
        }
        foreach ($this->foodVendorRepository->findAll() as $item) {
            $this->entityManager->remove($item);
        }
    }

    public function generate(int $vendorAmount = 3, int $orderAmount = 4, int $itemAmount = 10): void
    {
        $vendors = $this->generateVendors($vendorAmount);
        foreach ($vendors as $vendor) {
            $orders = $this->generateOrdersForVendor($vendor, $orderAmount);
            foreach ($orders as $order) {
                $this->generateItemsForOrder($order, $itemAmount);
            }
        }
        $this->entityManager->flush();
    }

    /**
     * @return FoodVendor[]
     */
    public function generateVendors(int $amount = 10): array
    {
        $vendors = [];
        foreach (range(1, $amount) as $i) {
            $vendor = new FoodVendor;
            $vendor->setName('Food Vendor ' . $i);
            $this->entityManager->persist($vendor);
            $vendors[] = $vendor;
        }
        return $vendors;
    }

    /**
     * @return FoodOrder[]
     */
    public function generateOrdersForVendor(FoodVendor $vendor, int $amount = 10): array
    {
        $orders = [];
        foreach (range(1, $amount) as $i) {
            $order = new FoodOrder;
            $order->setFoodVendor($vendor);
            if ($i % 2 === 0) {
                $order->close();
            }
            $this->entityManager->persist($order);
            $orders[] = $order;
        }
        return $orders;
    }

    /**
     * @return OrderItem[]
     */
    public function generateItemsForOrder(FoodOrder $order, int $amount = 10): array
    {
        $items = [];
        foreach (range(1, $amount) as $i) {
            $item = new OrderItem;
            $item->setName('Item ' . $i);
            $item->setFoodOrder($order);
            if ($i % 2 === 0) {
                $item->setExtras('Extra ' . $i);
            }
            $this->entityManager->persist($item);
            $items[] = $item;
        }
        return $items;
    }
}
